﻿namespace StringFormatting
{
    public static class Program
    {
        public static void Main()
        {
            FormattingSamples();
        }

        /// <summary>
        /// CheatSheet pro formaty stringu: http://slashview.com/file/20140509/.Net%20String%20Format%20Cheat%20Sheet.pdf
        /// </summary>
        private static void FormattingSamples()
        {
            Console.WriteLine("========= String interpolation =========");
            string hello = "hello";
            string world = "world";
            Console.WriteLine(hello + " " + world + "!");
            Console.WriteLine($"{hello} {world}!"); // C# 6.0 feature
            Console.WriteLine("========================================" + Environment.NewLine);

            Console.WriteLine("========= Verbatim strings =========");
            Console.WriteLine("Windows path: C:\\ProgramFiles\\VisualStudio (here we have to escape character '\\')");
            Console.WriteLine(@"Windows path: C:\\ProgramFiles\\VisualStudio (verbatim strings @""..."" take the content as-is)");
            Console.WriteLine(@"Windows path: C:\ProgramFiles\VisualStudio");
            Console.WriteLine("====================================" + Environment.NewLine);

            Console.WriteLine("========= Alignment =========");
            string hi = "hi";
            string name = "John";
            Console.WriteLine($"{hi}, {name}");
            Console.WriteLine($"{hi,2}, {name}"); // zarovna vpravo na 2 znaky
            Console.WriteLine($"{hi,4}, {name}"); // zarovna vpravo na 4 znaky
            Console.WriteLine($"{hi,10}, {name}"); // zarovna vpravo na 10 znakov
            Console.WriteLine($"{hi,-10}, {name}"); // zarovna VLEVO na 10 znakov
            Console.WriteLine("============================" + Environment.NewLine);

            Console.WriteLine("========= DateTime =========");
            // vypise aktualni datum, viz trida DateTime (podrobnosti o tride: najeti kurzoru nad tridu + stisk klavesy F12)
            Console.WriteLine("DateTime Now: " + DateTime.Now);
            // formatovani data: Monday, February, 2016
            Console.WriteLine("Today is {0:dddd}, {0:MMMM}, {0:yyy}", DateTime.Now);
            Console.WriteLine("============================" + Environment.NewLine);

            // see https://docs.microsoft.com/en-us/dotnet/standard/base-types/custom-numeric-format-strings?redirectedfrom=MSDN#Specifier0
            Console.WriteLine("========= Numbers =========");
            double d = 12.498798354;
            Console.WriteLine($"{d:00.00} (two decimal places)");
            Console.WriteLine($"{d:00.0000} (four decimal places)");
            Console.WriteLine($"{d:F4} (four decimal places - alt)");
            Console.WriteLine("============================" + Environment.NewLine);

            Console.WriteLine("========= Colors =========");
            Console.ForegroundColor = ConsoleColor.Green;
            Console.WriteLine("Green text");
            Console.ResetColor();
            Console.BackgroundColor = ConsoleColor.Red;
            Console.WriteLine("Red background");
            Console.ResetColor();
            Console.WriteLine("Normal text");
            Console.WriteLine("============================" + Environment.NewLine);
        }
    }
}